@@ -50,6 +50,7 @@ pub const STATUS_STAGE_FILE: KeyEvent = no_mod(KeyCode::Enter);
 pub const STATUS_STAGE_ALL: KeyEvent = no_mod(KeyCode::Char('a'));
 pub const STATUS_RESET_FILE: KeyEvent =
     with_mod(KeyCode::Char('D'), KeyModifiers::SHIFT);
+pub const DIFF_RESET_HUNK: KeyEvent = STATUS_RESET_FILE;
 pub const STATUS_IGNORE_FILE: KeyEvent = no_mod(KeyCode::Char('i'));
 pub const STASHING_SAVE: KeyEvent = no_mod(KeyCode::Char('s'));
 pub const STASHING_TOGGLE_UNTRACKED: KeyEvent =
@@ -57,7 +58,10 @@ pub const STASHING_TOGGLE_UNTRACKED: KeyEvent =
 pub const STASHING_TOGGLE_INDEX: KeyEvent =
     no_mod(KeyCode::Char('i'));
 pub const STASH_APPLY: KeyEvent = no_mod(KeyCode::Enter);
+pub const STASH_OPEN: KeyEvent = no_mod(KeyCode::Right);
 pub const STASH_DROP: KeyEvent =
     with_mod(KeyCode::Char('D'), KeyModifiers::SHIFT);
 pub const CMD_BAR_TOGGLE: KeyEvent = no_mod(KeyCode::Char('.'));
 pub const LOG_COMMIT_DETAILS: KeyEvent = no_mod(KeyCode::Enter);
+pub const COMMIT_AMEND: KeyEvent =
+    with_mod(KeyCode::Char('a'), KeyModifiers::CONTROL);
