@@ -12,6 +12,7 @@ pub static CMD_SPLITTER: &str = " ";
 
 pub static MSG_TITLE_ERROR: &str = "Error";
 pub static COMMIT_TITLE: &str = "Commit";
+pub static COMMIT_TITLE_AMEND: &str = "Commit (Amend)";
 pub static COMMIT_MSG: &str = "type commit message..";
 pub static STASH_POPUP_TITLE: &str = "Stash";
 pub static STASH_POPUP_MSG: &str = "type name (optional)";
@@ -19,6 +20,7 @@ pub static CONFIRM_TITLE_RESET: &str = "Reset";
 pub static CONFIRM_TITLE_STASHDROP: &str = "Drop";
 pub static CONFIRM_MSG_RESET: &str = "confirm file reset?";
 pub static CONFIRM_MSG_STASHDROP: &str = "confirm stash drop?";
+pub static CONFIRM_MSG_RESETHUNK: &str = "confirm reset hunk?";
 
 pub static LOG_TITLE: &str = "Commit";
 pub static STASHLIST_TITLE: &str = "Stashes";
@@ -99,6 +101,12 @@ pub mod commands {
         CMD_GROUP_DIFF,
     );
     ///
+    pub static DIFF_HUNK_REVERT: CommandText = CommandText::new(
+        "Revert hunk [D]",
+        "reverts selected hunk",
+        CMD_GROUP_DIFF,
+    );
+    ///
     pub static DIFF_HUNK_REMOVE: CommandText = CommandText::new(
         "Remove hunk [enter]",
         "removes selected hunk from stage",
@@ -148,6 +156,12 @@ pub mod commands {
         CMD_GROUP_COMMIT,
     );
     ///
+    pub static COMMIT_AMEND: CommandText = CommandText::new(
+        "Amend [^a]",
+        "amend last commit",
+        CMD_GROUP_COMMIT,
+    );
+    ///
     pub static STAGE_ITEM: CommandText = CommandText::new(
         "Stage Item [enter]",
         "stage currently selected file or entire path",
@@ -246,6 +260,12 @@ pub mod commands {
         "drop selected stash",
         CMD_GROUP_STASHES,
     );
+    ///
+    pub static STASHLIST_INSPECT: CommandText = CommandText::new(
+        "Inspect [\u{2192}]", //→
+        "open stash commit details (allows to diff files)",
+        CMD_GROUP_STASHES,
+    );
 
     ///
     pub static LOG_DETAILS_TOGGLE: CommandText = CommandText::new(
